Package: rSFA
Maintainer: Martin Zaefferer <martin.zaefferer@gmx.de>
License: GPL (>= 2)
Title: Slow Feature Analysis
Type: Package
Author: Wolfgang Konen <wolfgang.konen@fh-koeln.de>, Martin Zaefferer,
    Patrick Koch; Bug hunting and testing by Ayodele Fasika, Ashwin
    Kumar, Prawyn Jebakumar
Description: Slow Feature Analysis (SFA), ported to R based on
    'matlab' implementations of SFA: 'SFA toolkit' 1.0 by Pietro Berkes and 'SFA toolkit'
    2.8 by Wolfgang Konen.
Version: 1.5
Date: 2022-03-29
Depends: R (>= 2.0.0)
Imports: stats, MASS, graphics, grDevices
NeedsCompilation: no
Repository: CRAN
RoxygenNote: 7.1.2
Packaged: 2022-03-29 09:40:39 UTC; Martin
Date/Publication: 2022-03-29 10:00:07 UTC
Built: R 4.6.0; ; 2026-04-27 02:26:27 UTC; unix
