# Generated by roxygen2: do not edit by hand

export(addNoisyCopies)
export(customRepmat)
export(customSize)
export(etaval)
export(gaussClassifier)
export(gaussCreate)
export(gaussLoad)
export(gaussSave)
export(lcovCreate)
export(lcovFix)
export(lcovPca)
export(lcovPca2)
export(lcovTransform)
export(lcovUpdate)
export(nlDim)
export(nlExpand)
export(sfa1)
export(sfa1Create)
export(sfa1Step)
export(sfa2)
export(sfa2Create)
export(sfa2Step)
export(sfaBSh)
export(sfaClassPredict)
export(sfaClassify)
export(sfaExecute)
export(sfaExpand)
export(sfaLoad)
export(sfaNlRegress)
export(sfaPBootstrap)
export(sfaPreproc)
export(sfaSave)
export(sfaStep)
export(sfaTimediff)
export(xpDim)
importFrom(MASS,ginv)
importFrom(grDevices,rainbow)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(stats,filter)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
